/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../../libs.h"
#include "../../assert/assert.h"
#include "../../engine/event.h"
#include "../shaderprogram.h"
#include "../../file/assetmanager.h"

#define STANDARD_SHADER_UNIFORM_NAME_VIEW "u_View"
#define STANDARD_SHADER_UNIFORM_NAME_PROJ "u_Proj"
#define STANDARD_SHADER_UNIFORM_NAME_TEXT "u_Text"
#define STANDARD_SHADER_UNIFORM_NAME_MODEL "u_Model"
#define STANDARD_SHADER_UNIFORM_NAME_COLOR "u_Color"

#define STANDARD_SHARER_FILE_VERTEX "shaders/textured.vert"
#define STANDARD_SHADER_FILE_FRAGMENT "shaders/textured.frag"

#define STANDARD_SHADER_SHADER(ss) (&ss->shaderProgram->data.shader.shader)

typedef struct {
  assetmanageritem_t *shaderProgram;
  assetmanagerholder_t holder;
  eventlistener_t *onItemLoaded;

  shaderuniform_t uniformView;
  shaderuniform_t uniformProjection;
  shaderuniform_t uniformPosition;
  shaderuniform_t uniformTexture;
  shaderuniform_t uniformColor;
} standardshader_t;

/**
 * Initialize the standard shader.
 * 
 * @param stds Standard Shader Instance.
 * @param assetManager Asset Manager to load the shaders' assets.
 */
void standardShaderInit(standardshader_t *stds, assetmanager_t *assetManager);

/**
 * Bind the standard shader as the currently active shader.
 * 
 * @param stds Standard shader to bind.
 */
void standardShaderUse(standardshader_t *stds);

/**
 * Set the camera for the standard shader.
 * 
 * @param stds Standard shader instance.
 * @param camera Camera to bind.
 */
void standardShaderSetCamera(standardshader_t *stds, camera_t *camera);

/**
 * Set the texture for the standard shader.
 * 
 * @param stds Standard shader instance.
 * @param texture Texture to bind.
 */
void standardShaderSetTexture(standardshader_t *stds, texture_t *texture);

/**
 * Set the color for the standard shader.
 * 
 * @param stds Standard shader instance.
 * @param color Color to bind.
 */
void standardShaderSetColor(standardshader_t *stds, pixel_t color);

/**
 * Set the position matrix for the standard shader.
 * 
 * @param stds Standard shader to use.
 * @param matrix Matrix to use.
 */
void standardShaderSetPosition(standardshader_t *stds, matrix_t *matrix);

/**
 * Dispose/cleanup the standard shader.
 * 
 * @param stds Standard shader instance.
 * @param assMan Asset manager instance.
 */
void standardShaderDispose(standardshader_t *stds, assetmanager_t *assMan);