/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "save.h"

void saveManagerInit(savemanager_t *man) {
  ASSERT_NOT_NULL(man);

  man->count = 0;
  eventManagerInit(&man->events);

  // Set up defaults
  saveManagerSetU8(man, SAVE_KEY_TEXTURE_SCALE, 0x00);
}

savevalue_t * saveManagerAddOrGet(savemanager_t *man, const char key[]) {
  uint8_t i, j;

  ASSERT_NOT_NULL(man);
  ASSERT_NOT_NULL(key);

  j = saveManagerGetKey(man, key);
  ASSERT_IF(j >= SAVE_VALUES_MAX, 0xFF);
  ASSERT_IF(j < SAVE_VALUES_MAX, ASSERT_LESS_THAN(j, SAVE_VALUES_MAX));
  if(j != 0xFF) return man->values + j;

  // Find empty key
  for(i = 0; i < man->count; i++) {
    if(man->keys[i] != NULL) continue;

    man->keys[i] = key;
    return man->values + i;
  }

  // No empty key
  man->keys[man->count] = key;
  return man->values + man->count++;
}

uint8_t saveManagerGetKey(savemanager_t *man, const char key[]) {
  ASSERT_NOT_NULL(man);
  return (uint8_t) arrayFindString(man->keys, man->count, key);
}

void saveManagerRemove(savemanager_t *man, const char key[]) {
  uint8_t i;

  ASSERT_NOT_NULL(man);
  ASSERT_NOT_NULL(key);

  i = saveManagerGetKey(man, key);
  if(i == 0xFF) return;

  man->keys[i] = NULL;
}

/////////////////////////////////// Setters ////////////////////////////////////

savevalue_t * saveManagerSetBool(
  savemanager_t *man, const char key[], bool val
) {
  savevalue_t *v = saveManagerAddOrGet(man, key);
  v->b = val;

  void *evtArgs[3] = { man, key, &val };
  eventManagerTrigger(&man->events, SAVE_EVENT_VALUE_CHANGED, evtArgs, 3);

  return v;
}

savevalue_t * saveManagerSetU8(
  savemanager_t *man, const char key[], uint8_t val
) {
  savevalue_t *v = saveManagerAddOrGet(man, key);
  v->u8 = val;

  void *evtArgs[3] = { man, key, &val };
  eventManagerTrigger(&man->events, SAVE_EVENT_VALUE_CHANGED, evtArgs, 3);

  return v;
}

/////////////////////////////////// Getters ////////////////////////////////////

bool saveManagerGetBool(savemanager_t *man, const char key[]) {
  return saveManagerAddOrGet(man, key)->b;
}

uint8_t saveManagerGetU8(savemanager_t *man, const char key[]) {
  return saveManagerAddOrGet(man, key)->u8;
}