/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../libs.h"
#include "../assert/assert.h"
#include "../util/array.h"
#include "../engine/event.h"

#define SAVE_VALUES_MAX 200

#define SAVE_KEY_TEXTURE_SCALE "TEXTURE_SCALE"

#define SAVE_EVENT_VALUE_CHANGED 0x00

typedef union {
  bool b;
  uint8_t u8;
  int8_t i8;
  int16_t i16;
  int32_t i32;
} savevalue_t;

typedef struct {
  const char *keys[SAVE_VALUES_MAX];
  savevalue_t values[SAVE_VALUES_MAX];
  uint8_t count;
  eventmanager_t events;
} savemanager_t;

/**
 * Initialize the save manager.
 * 
 * @param man Save manager to initialize.
 */
void saveManagerInit(savemanager_t *man);

/**
 * Either add, or return, a pointer to a specific save manager value.
 * 
 * @param man Save manager instance.
 * @param key Key to get (or add).
 * @return A pointer to the save value.
 */
savevalue_t * saveManagerAddOrGet(savemanager_t *man, const char key[]);

/**
 * Get the index within the save manager that the given key is at. Returns 0xFF
 * if the key is not present within the save manager.
 * 
 * @param man Save manager to check.
 * @param key Key to check for.
 * @return The index within the save manager that the key is at.
 */
uint8_t saveManagerGetKey(savemanager_t *man, const char key[]);

/**
 * Remove a key from the save manager.
 * 
 * @param man Save manager to remove the key from.
 * @param key Key to remove.
 */
void saveManagerRemove(savemanager_t *man, const char key[]);

/////////////////////////////////// Setters ////////////////////////////////////

/**
 * Setter for Save Manager.
 * 
 * @param man Save Manager instance.
 * @param key Key to set.
 * @param val Value to set.
 * @return The save value pointer.
 */
savevalue_t * saveManagerSetBool(savemanager_t *man,const char key[],bool val);
savevalue_t * saveManagerSetU8(savemanager_t *man,const char key[],uint8_t val);

/////////////////////////////////// Getters ////////////////////////////////////

/**
 * Getter for the Save Manager.
 * 
 * @param man Save Manager to get from.
 * @param key Key to get.
 * @return The value for that key, may be garbage if not previously set.
 */
bool saveManagerGetBool(savemanager_t *man, const char key[]);
uint8_t saveManagerGetU8(savemanager_t *man, const char key[]);