/**
 * Copyright (c) 2022 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "scenemanager.h"

bool _sceneManagerItemFilter(int32_t i, void *item) {
  return ((sceneitem_t *)item)->onRender != NULL;
}

void sceneManagerInit(scenemanager_t *sceneManager) {
  ASSERT_NOT_NULL(sceneManager);

  sceneManager->itemCount = 0;
}

sceneitem_t * sceneManagerAddItem(scenemanager_t *man, scenerender_t *onRender) {
  sceneitem_t *item;

  ASSERT_NOT_NULL(man);
  ASSERT_NOT_NULL(onRender);
  ASSERT_GREATER_THAN_EQUAL_TO(man->itemCount, 0);
  ASSERT_LESS_THAN(man->itemCount, SCENE_ITEM_COUNT_MAX);

  item = man->items + man->itemCount++;
  item->onRender = onRender;

  return item;
}

void sceneManagerRemoveItem(scenemanager_t *man, sceneitem_t *item) {
  ASSERT_NOT_NULL(man);
  ASSERT_NOT_NULL(item);
  item->onRender = NULL;
}

void sceneManagerRestack(scenemanager_t *man) {
  ASSERT_NOT_NULL(man);
  arrayFilter(
    sizeof(sceneitem_t), man->items, &man->itemCount, &_sceneManagerItemFilter
  );
}

void sceneManagerClear(scenemanager_t *man) {
  ASSERT_NOT_NULL(man);
  man->itemCount = 0;
}